/*
  * Originates in the work for a seminar in the winter semester 2000/2001
  * at the University of Essen (http://www.uni-essen.de), department:
  * "MIS and software development" (http://nestroy.wi-inf.uni-essen.de).
  *
  * This DLL registers the function "BSF" with Rexx;
  * it allows Java to call into Rexx using the fine RexxSAA-interface
  * conventions and JNI ("Java Native Interface").
  *
  * version: 1.0.3, 2001-04-25
  *
  * author:  Rony G. Flatscher, University of Essen, WU  (pronounced: "vey-u") University Vienna,
  *          Peter Kalender (Oct 2000 to Feb 2001), University of Essen
*/
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#define INCL_RXSHV
#define INCL_RXFUNC

#ifdef USE_REXXTRANS
# include <rexxtrans.h>
#elif defined USE_OREXX
# include <rexx.h>
#else
# include <rexxsaa.h>
#endif

#ifdef _MSC_VER
# pragma warning(disable:4100)
#endif


extern "C"
{
APIRET APIENTRY
 BSF (PSZ name, ULONG argc, PRXSTRING argv, PSZ queuename, PRXSTRING retstr);
// RexxFunctionHandler BSF (PSZ name, ULONG argc, PRXSTRING argv, PSZ queuename, PRXSTRING retstr);
}


#define RFH_BSF         "BSF"

#define DLLNAME         "BSF4Rexx"/* name of DLL and name for "PARSE SOURCE"*/


        // Java support
#include <jni.h>                /* "Java Native Interface" (JNI)           */
        // generated by Java to support the JNI
        // see comments at the end of "RexxAndJava.java"
#include "com_ibm_bsf_engines_rexx_RexxAndJava.h"

#define JAVA_CALL_BSF   "javaCallBSF"/* name of Java method to carry out the BSF-call*/



JNIEnv *globEnv;        //used by BSF() to call back to Java
jobject globObj;        //used by BSF() to call back to Java


/************************************************************
 * 'BSF' calls back to the Java method 'javaCallBSF'
 ************************************************************/
APIRET APIENTRY
BSF (PSZ name,
                  ULONG argc,
                  PRXSTRING argv,
                  PSZ queuename,
                  PRXSTRING retstr)
     {

        // get Java callback method
           jclass cls = (*globEnv).GetObjectClass(globObj);

        // lookup method, signature: one argument (a String array), returning a String
        jmethodID mid = (*globEnv).GetMethodID(cls, JAVA_CALL_BSF,
                                                "([Ljava/lang/String;)Ljava/lang/String;");

        if (mid == 0) return 90l;/* hinting at "External name not found!", subcode 998 would be "method"*/

//    if (mid == 0) return 40;    // hinting at "Incorrect call to routine!"
//    if (mid == 0) return 43;    // hinting at "Routine not found!"
//    if (mid == 0) return 97;    // hinting at "Object method not found!"

        //convert arguments array to Java format
        jobjectArray arr = (*globEnv).NewObjectArray (
                                      argc,
                                    (*globEnv).FindClass("java/lang/String"),
                                    (*globEnv).NewStringUTF("")
                                                     );


        // RXSTRINGs passed from Rexx have an additional \0 appended according Object Rexx 2.1 docs
        for(ULONG i=0; i<argc; i++)
        {
            if (argv[i].strptr==NULL)   // NULL (argument missing) ?
            {
               (*globEnv).SetObjectArrayElement(arr, i, NULL);
            }
            else                        // argument as a string in hand
            {
               (*globEnv).SetObjectArrayElement(arr, i,
                                               (*globEnv).NewStringUTF(argv[i].strptr));
            }
        }

        //call Java method and create Rexx return value
        jstring ReturnValue = (jstring)(*globEnv).CallObjectMethod(globObj, mid, arr);

        retstr->strlength = 0;          // indicate no chars available

        if (ReturnValue!=NULL)          // value returned from Java?
        {
                const char *str = globEnv->GetStringUTFChars(ReturnValue, 0);
                int         len = strlen(str);
                retstr->strptr  = new char[len+1];
                strcpy(retstr->strptr, str);
                retstr->strlength = len;
                globEnv->ReleaseStringUTFChars(ReturnValue, str);
        }
        else    // o.k. Java returned null
        {
// rgf, 2001-05
if (retstr->strptr != NULL)
  {
// printf("/// BSF4Rexx.cpp:BSF(): freeing memory of retstr->strptr ! \\\\\\");


#ifdef USE_REGINA
// printf("              'USE_REGINA' defined, RexxFreeMemory()\n");
        RexxFreeMemory(retstr->strptr);

#elif defined WIN32
// printf("              'WIN32' defined, GlobalFree()\n");
        GlobalFree(retstr->strptr);

#else
// printf("              'WIN32' *not* defined, free()\n");
        free(retstr->strptr);
#endif

  }                                                 /* End if, rgf, 2001-05*/

//           GlobalFree(retstr->strptr);     // free memory allocated by Rexx;

// but this bombs eventually with at least JRE 1.3.0C, where Java aborts with the message that
// an illegal access to the memory was done outside of the JVM;

           retstr->strptr    = NULL;/* set string pointer to NULL (as if no value was returned)*/
           retstr->strlength = 0;          // indicate no chars available
        }

        if ( globEnv->ExceptionCheck() )       /* pending exception in JVM!*/
          {
             globEnv->ExceptionDescribe();      // print the stack
             globEnv->ExceptionClear();       /* clear exception in the JVM*/
             return -40;/* interrupt Rexx with "Incorrect call to routine!"*/
          }
        return 0l;
     }



/************************************************************
 * 'RegisterBSF' registers the 'CallBSF' method with the
 * Rexx interpreter
 ************************************************************/
JNIEXPORT jlong JNICALL
Java_com_ibm_bsf_engines_rexx_RexxAndJava_jniRegisterBSF (JNIEnv *env, jobject obj)
     {
         LONG        ReturnValue;
         ReturnValue = RexxRegisterFunctionDll(RFH_BSF , DLLNAME, RFH_BSF );
         return      ReturnValue;
     }



/*
 * Class:     com_ibm_bsf_engines_rexx_RexxAndJava
 * Method:    StartWithArgs
 * Signature: (Ljava/lang/String;[Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_ibm_bsf_engines_rexx_RexxAndJava_jniRexxStart (JNIEnv *env,
                              jobject         obj,
                              jstring         source,
                              jobjectArray    array)
     {
         // set global variables for callbacks to Java (via BSF())
         globEnv = env;
         globObj = obj;

         //convert Java string containing script to Rexx string
         const char      *str = env->GetStringUTFChars(source, 0);
         char      *sourceStr = new char[strlen(str)+1];
         strcpy(sourceStr, str);
         env->ReleaseStringUTFChars(source, str);

         // create argument array
         PRXSTRING argList = NULL;       // pointer to array of RXSTRINGS
         jsize   arrNum = 0;
         long    numArgs= 0l;

         if (array != NULL)              // an Object array available?
           {
               arrNum    =  env->GetArrayLength(array);
               numArgs   = (long) arrNum;
           }

         if (arrNum>0)
           {
             argList = new RXSTRING [ arrNum ];
             for (int i = 0; i < arrNum; ++i)
               {
                  if ((env->GetObjectArrayElement(array, i))==NULL)/* no argument*/
                    {
                       argList[i].strptr   =NULL;
                       argList[i].strlength=0;
                    }
                  else        /* a string argument in hand, give it to Rexx*/
                    {
                       jstring        javaStr = (jstring) env->GetObjectArrayElement(array, i);
                       const char     *strArg = env->GetStringUTFChars(javaStr, 0);
                       int                len = strlen(strArg);
                       char        *tmpString = new char[len+1];
                       strcpy(tmpString, strArg);
                       env->ReleaseStringUTFChars(javaStr, strArg);

                       MAKERXSTRING( argList[i], tmpString, len);/* save arg for Rexx*/
                    }
               }
           }

         RXSTRING        RXsourceStr[2];
         MAKERXSTRING   (RXsourceStr[0], sourceStr, strlen(sourceStr));
         PRXSTRING       Instore = &RXsourceStr[0];
         MAKERXSTRING   (RXsourceStr[1], NULL, 0);

         SHORT           ReturnCode;
         LONG            ReturnValue;
         RXSTRING        Result;         // let Rexx allocate buffer
         MAKERXSTRING   (Result, NULL, 0);

         PSZ             ProgramName = DLLNAME ;

         //start Rexx
         ReturnValue = RexxStart(
                         arrNum,         // LONG         ArgCount
                         argList,        // PRXSTRING    ArgList
                         ProgramName,    // PSZ          ProgramName
                         Instore,        // PRXSTRING    Instore
                         NULL,           // PSZ          EnvName
                         RXSUBROUTINE,   // LONG         CallType
                         NULL,           // PRXSYSEXIT   Exits
                         &ReturnCode,    // PUSHORT      ReturnCode
                         &Result         // PRXSTRING    Result
                         );


// printf("BSF4Rexx.cpp:Java_com_ibm_bsf_engines_rexx_RexxAndJava_jniRexxStart()\n");
// printf("              about to free RXSourceStr[1].strptr!\n");
// printf("              pointing to: %p\n", RXsourceStr[1].strptr);
// printf("              pointing to: %p\n", (RXsourceStr+1)->strptr);
// _flushall();



#ifdef USE_REGINA
// printf("              'USE_REGINA' defined, RexxFreeMemory()\n");
        RexxFreeMemory(RXsourceStr[1].strptr);

#elif defined WIN32
// printf("              'WIN32' defined, GlobalFree()\n");
        GlobalFree(RXsourceStr[1].strptr);

#else
// printf("              'WIN32' *not* defined, free()\n");
        free(RXsourceStr[1].strptr);       // free tokenized image
#endif



        if (ReturnValue != 0l)     // error occurred, return null
        {
// printf("BSF4Rexx.cpp:Java_com_ibm_bsf_engines_rexx_RexxAndJava_jniRexxStart()\n");
// printf("              returning *direct* NULL to Java !\n");
// _flushall();

#ifdef USE_REGINA
// printf("              'USE_REGINA' defined, RexxFreeMemory()\n");
        RexxFreeMemory(Result.strptr);

#elif defined WIN32
// printf("              'WIN32' defined, GlobalFree()\n");
        GlobalFree(Result.strptr);

#else
// printf("              'WIN32' *not* defined, free()\n");
           free(Result.strptr);       // free tokenized image
#endif
           return NULL;
        }

 // --- get result and return it to Java
        jstring jResult=NULL;               /* define Java String to return*/

          if ( Result.strptr != NULL )           // a result was returned
            {
               char *resString;
               if (Result.strlength==0)  // empty string?
                 {
                   jResult=env->NewStringUTF("");
                 }
               else                              // valid non-empty string
                 {
                   resString = new char[Result.strlength+1];
                   strncpy(resString, Result.strptr, Result.strlength);
//                   resString[Result.strlength]='\0';
                   resString[Result.strlength]=0;       // rgf was here
                   jResult=env->NewStringUTF(resString);
                   delete []resString;
                 }
// printf("BSF4Rexx.cpp:Java_com_ibm_bsf_engines_rexx_RexxAndJava_jniRexxStart()\n");
// printf("              jResult has value, before releasing 'strptr' memory...\n");
// _flushall();

#ifdef USE_REGINA
// printf("              'USE_REGINA' defined, RexxFreeMemory()\n");
        RexxFreeMemory(Result.strptr);

#elif defined WIN32
// printf("              'WIN32' defined, GlobalFree()\n");
        GlobalFree(Result.strptr);


#else
// printf("              'WIN32' *not* defined, free()\n");
               free(Result.strptr);
#endif
            }

// if (jResult==NULL)
//  {
// printf("BSF4Rexx.cpp:Java_com_ibm_bsf_engines_rexx_RexxAndJava_jniRexxStart()\n");
// printf("              jResult == NULL, returning NULL to Java !\n");
// _flushall();
//  }                                                               /* End if*/

         return jResult;
     }



